import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class RKdemo extends Frame {
    public RK_resource_set[] arrayOfRS;
    public RKdemoCanvas[] arrayOfCanvas;
    public int num;
    public Font demoFont;

    public RKdemo() {
	super("RKdemo");

	num = 0;
	arrayOfRS = new RK_resource_set[3];
	arrayOfCanvas = new RKdemoCanvas[3];

	demoFont = new Font("SansSerif", Font.BOLD, 14);
	
	// make Canvas
	arrayOfCanvas[0] = new RKdemoCanvas(this, "Reserve 1");
	arrayOfCanvas[1] = new RKdemoCanvas(this, "Reserve 2");
	arrayOfCanvas[2] = new RKdemoCanvas(this, "Reserve 3");

	// Add winodws
	setLayout(new GridLayout(3,0));	// as big as possible
	this.add(new RKdemoPanel(this, arrayOfCanvas[0]));
	this.add(new RKdemoPanel(this, arrayOfCanvas[1])); 
	this.add(new RKdemoPanel(this, arrayOfCanvas[2]));

	// Set our initial size and pop the window up.
	this.setSize(500, 600);
	this.show();
    }

    void getRS(int id) 
    {
	try {
	    RK_resource_set rs = new RK_resource_set(id);
	    rs.readStatus();
	    arrayOfRS[num++] = rs;
	    System.out.println(num+": ID("+Integer.toHexString(id)+")");
	} catch (IOException e) {
	    System.out.println("Failed to read ID("+Integer.toHexString(id)+")");
	}
    }

    public static void main(String[] args)
    {
	RKdemo demo = new RKdemo();

	for (int i = 0; i < args.length; i++) {
	    System.out.println("arg("+i+"): "+args[i]);

	    int id = (Long.valueOf(args[i], 16)).intValue();

	    System.out.println("ID("+Integer.toHexString(id)+")");

	    demo.getRS(id);
	}

	RKdemoThread th = new RKdemoThread(demo);
	th.start();
    }
}

class RKdemoThread extends Thread {
    RKdemo demo;

    public RKdemoThread(RKdemo main) {
	super("RKdemo");
	demo = main;
    }
    public void run() {
	while (true) {
	    for (int j = 0; j < 60; j++) {
		for (int i = 0; i < demo.num; i++) {
		    try {
			demo.arrayOfRS[i].readStatus();
			int u = demo.arrayOfRS[i].utilized;
			System.out.println(i +": util("+ u +")"
					   +" enf_mode("+ demo.arrayOfRS[i].enf_mode +")");
			
			demo.arrayOfCanvas[i].newPoint(j, u);
		    } catch (Exception e) {
		    }
		}
		try {
		    sleep(1000);	// milli seconds
		} catch (InterruptedException e) {}
	    }
	}
    }
}

class RKdemoPanel extends Panel {
    RKdemo demo;
    RKdemoCanvas canvas;

    public RKdemoPanel(RKdemo d, RKdemoCanvas c) {
	super();

	demo = d;
	canvas = c;

	setLayout(new BorderLayout());

	Button b = new Button(c.title);
	b.setFont(demo.demoFont);
	add("North", b);
	add("Center", c);
    }

    public Insets getInsets() {
	return new Insets(4,4,5,5);
    }

    public void paint(Graphics g) {
	Dimension d = getSize();
	Color bg = getBackground();

	g.setColor(bg);
	g.draw3DRect(0, 0, d.width-1, d.height-1, true);
	g.draw3DRect(3, 3, d.width-7, d.height-7, false);

    }
}

class RKdemoCanvas extends Canvas {
    RKdemo demo;
    public String title;
    Dimension dimCanvas;	// width and height
    private Vector points = new Vector(60);

    public RKdemoCanvas(RKdemo d, String name) {
	super();

	title = name;
	demo = d;
    }

    // draw tilte and all lines
    public void paint(Graphics g) {
	int i, s;
	demoPoint p1, p2;

	// get the Canvas size
	dimCanvas = getSize();

	// clear area
	g.clearRect(0, 0, dimCanvas.width, dimCanvas.height);

	// draw lines
	s = points.size();
	if (s == 0) {
	    return;
	}
	p1 = (demoPoint)points.elementAt(0);
	for (i = 0; i < s; i++) {
	    p2 = (demoPoint)points.elementAt(i);
	    g.drawLine(rX(p1.x), rY(p1.y), rX(p2.x), rY(p2.y));
	    p1 = p2;
	}
    }

    // add a new point and draw a line
    public void newPoint(int x, int y) {
	int i;
	demoPoint p;
	Graphics g = getGraphics();

	if (x < 0 || x > 59) {
	    System.err.println("RKdemoCanvas.newPoint: invalid point ("+x+","+y+")");
	    return;
	}
	if (x == 0) {
	    points.removeAllElements();
	    points.addElement(new demoPoint(x, y));
	    // redraw all
	    paint(g);
	    return;
	}

	points.addElement(new demoPoint(x, y));
	i = points.size();
	p = (demoPoint)points.elementAt(i-2);
	g.drawLine(rX(p.x), rY(p.y), rX(x), rY(y));
    }

    // conver data to Canvas coordinations
    private int rX(int x) {
	return (dimCanvas.width*x)/60;
    }
    private int rY(int y) {
	return dimCanvas.height - (dimCanvas.height*y)/100;
    }

    // private class
    class demoPoint {
	public int x, y;
	public demoPoint(int x, int y) {
	    this.x = x;
	    this.y = y;
	}
    }
}

class RK_resource_set {
    public int id;
    public String dir;
    public int enf_mode, rep_mode;
    public int reserved, utilized;

    RK_resource_set(int i) 
    {
	id = i;
	// make a string of /proc/rk/<id> where id is hex
	dir = "/proc/rk/"+Integer.toHexString(id);
	System.out.println("RK_resource_set: "+dir);
    }

    void readStatus()
	throws IOException
    {
	DataInputStream in;
	// open "/proc/rk/<id>/cpu"
	in = new DataInputStream(new FileInputStream(dir+"/cpu.raw"));
	enf_mode = in.readInt();
	rep_mode = in.readInt();
	reserved = in.readInt();
	utilized = in.readInt();
	// close
	in.close();
    }
}
