/*
 * Evaluation:
 *	<load> msec / 1 sec
 *	20 msec work and (approx) 10 msec sleep
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <rk/rk.h>
#include <signal.h>

long long	dummy_result;
rk_resource_set_t	rs;

void sigint_handler(int sig)
{
	rk_resource_set_destroy(rs);
	fprintf(stderr, "resource set destroyed successfully!\n");
	exit(0);
}

void
usleep(unsigned long t)
{
	struct timespec	s, r;
	s.tv_sec = 0, s.tv_nsec = t*1000;
	nanosleep(&s, &r);
}

main(int argc, char *argv[])
{
	rk_reserve_t	cpu_rsv;
	struct tm	*tmp;
	struct tm	ltn, ltnp;
	time_t	tn;
	int	who;
	int	start = 0;
	int	load = 200;
	struct timespec	c, t;
	int	enf_mode = RSV_SOFT;
	char	*com;

	com = rindex(argv[0], '/');
	if (com == NULL) {
		com = argv[0];
	} else {
		com++;
	}
	printf("I'm %s\n", com);

	if (argc == 1) {
		if (!strcmp(com, "eval_qos_hr1")) {
			who = 0;
			load = 100;
			t.tv_sec = 1, t.tv_nsec = 0;
			enf_mode = RSV_HARD;
		} else if (!strcmp(com, "eval_qos_hr2")) {
			who = 1;
			load = 100;
			t.tv_sec = 1, t.tv_nsec = 0;
			enf_mode = RSV_HARD;
		} else if (!strcmp(com, "eval_qos_sr1")) {
			who = 2;
			load = 50;
			t.tv_sec = 1, t.tv_nsec = 0;
			enf_mode = RSV_SOFT;
		} else if (!strcmp(com, "eval_qos_test")) {
		    	who = 3;
			load = 50;
			t.tv_sec = 1, t.tv_nsec = 0;
			enf_mode = RSV_HARD;
		} else {
			printf("I'm %s.\n", com);
			exit(1);
		}
	} else {
		printf("I'm not here.\n");
		exit(1);
	}

	/* create a resource set */
	rs = rk_resource_set_create();

	/*
	signal(SIGINT, sigint_handler);
	signal(SIGKILL, sigint_handler);
	*/

	/* create cpu reserve */
	{
		rk_reserve_param_data_t	p;
		c.tv_sec = 0, c.tv_nsec = load*1000*1000;
		p.enf_mode = enf_mode;
		p.sch_mode = p.rep_mode = RSV_SOFT;
		cpu_rsv = rk_cpu_reserve_create(rs, &c, &t, &p);
	}
	/* attach the current process to a resource set */
	rk_resource_set_attach_process(rs, getpid());

	printf("Resource Set (0x%x) CPU Reserve (0x%x)\n", (int)rs, (int)cpu_rsv);

	usleep(50*1000);		/* 50 mili sec */

	tn = time(NULL);
	tmp = localtime(&tn);
	ltnp = *tmp;
	for (;;) {
		cpu_tick_data_t	last, now;
		/* do some work */
		rdtsc(&last);	now = last;
		while ((now-last) < (200*1000)*20)	/* (200*100) = 1 mili sec w/ 200MHz CPU */
		{					/* ... 20 mili sec work */
			long long	l0, l1, l2;
			l0 = now-last;
			l1 = now*last;
			l2 = now/last;
			dummy_result = l0+l1+l2;
			rdtsc(&now);
		}

		/* check time */
		tn = time(NULL);
		tmp = localtime(&tn);
		ltn = *tmp;
		if (ltnp.tm_sec != ltn.tm_sec) {
			printf("who(%d) turn(%d) sec(%d)\n",
			       who, start, ltn.tm_sec);
			if (ltn.tm_sec == 0) {
				start++;
			}

			switch(who) {
			case 0:
				break;
			case 1:
				if (ltn.tm_sec == 9) {
					load = 150;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
				}
				if (ltn.tm_sec == 19) {
					load = 200;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
				}
				if (ltn.tm_sec == 29) {
					load = 250;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
				}
				if (ltn.tm_sec == 39) {
					load = 300;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
				}
				if (ltn.tm_sec == 49) {
					load = 250;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
				}
				if (ltn.tm_sec == 59) {
					load = 200;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
				}
				break;
			case 2:
				break;
			case 3:
				switch(ltn.tm_sec) {
				case 9:
				case 29:
				case 49:
					load = 100;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
					break;
				case 19:
				case 39:
				case 59:
					load = 50;
					c.tv_sec = 0, c.tv_nsec = load*1000*1000;
					rk_cpu_reserve_ctl(rs, cpu_rsv, &c, &t);
					printf("QoS\n");
					break;
				default:
					break;
				}
				break;
			default:
				break;
			}
		}
		ltnp = ltn;
		/*usleep(10*1000);			/* sleep for 10 mili sec */
	}
}
