#ifndef	RK_RK_H
#define	RK_RK_H

#undef	DEBUG_RK	/* enable debug code */
#undef	EVAL_RK		/* enable evaluation code */
#undef	UPERF_RK	/* enable micro benchmark code */

#ifdef	UPERF_RK
/**/
#undef	EVAL_RK		/* disable evaluation code */
#undef	DEBUG_RK	/* disable debug code */
/**/
extern	int	uperf_rk_isa_timer_programmed;
#endif

#ifdef DEBUG_RK
/**/
#if 1
#undef	DEBUG_RK_ENFORCE
#undef	DEBUG_RK_ENFORCE_NOT
#else
#define	DEBUG_RK_ENFORCE	if (debug_rk_enforce_flag)
#define	DEBUG_RK_ENFORCE_NOT	if (!debug_rk_enforce_flag)
extern	int	debug_rk_enforce_flag;
#endif
/**/
#endif /*DEBUG_RK*/

/*
 * types and defines
 */
typedef	unsigned long long *	cpu_tick_t;
typedef	unsigned long long	cpu_tick_data_t;

#define	NANOSEC			1000000000L
#define	MICROSEC		1000000L

static inline void
rk_rdtsc(cpu_tick_t data_p)
{
	__asm __volatile(
		"rdtsc"
		:"=a"(*(int *)(data_p)), "=d"(*(((int *)data_p)+1))
		:
		:"eax","edx");
}

/*
 * CPU reserve argument parameter
 */
enum rk_reserve_mode
{
	RSV_HARD	=0x1,
	RSV_FIRM	=0x2,
	RSV_SOFT	=0x4,
};
typedef enum rk_reserve_mode	rk_reserve_mode_t;

struct rk_reserve_param {
	rk_reserve_mode_t	sch_mode;	/* scheduling */
	rk_reserve_mode_t	enf_mode;	/* enforcement */
	rk_reserve_mode_t	rep_mode;	/* replenishment */
};
typedef	struct rk_reserve_param	rk_reserve_param_data_t;
typedef	struct rk_reserve_param	*rk_reserve_param_t;


#ifdef	__KERNEL__
/*
 * resource and reserve types
 */
enum rk_reserve_type
{
	RSV_NULL	=0x0,
	RSV_CPU		=0x1,
	RSV_NET,
	RSV_MEM,
	RSV_DEV,
};
typedef	enum rk_reserve_type	rk_reserve_type_t;

/*
 *	NULL			not running
 *	RUNNING			running
 *	DEPLETED		not running and depleted
 *	RUNNING|DEPLETED	running but depleted
 */
enum rk_reserve_state
{
	RSV_IS_NULL	=0x0,	/* not running */
	RSV_IS_RUNNING	=0x1,	/* actually running */
	RSV_IS_DEPLETED	=0x10,	/* it's empty */
};
typedef	enum rk_reserve_state	rk_reserve_state_t;

enum rk_timer_type
{
	TMR_NULL	=0x0,
	TMR_REPLENISH_RSV,	/* used for replenishing reserves */
};
typedef	enum rk_timer_type	rk_timer_type_t;

/*
 * Typedefs
 */
typedef struct rk_resource_set	*rk_resource_set_t;
typedef struct rk_reserve	*rk_reserve_t;
typedef	struct rk_timer		*rk_timer_t;

/*
 * struct rk_resource_set
 */
struct rk_resource_set {
	/* */
	rk_reserve_t		rs_cpu;		/* cpu */
	struct list_head	rs_devices;	/* list of device resources */
	/* */
	struct list_head	rs_rs;		/* list of resource_sets */
	/* */
#ifdef	linux
	struct list_head	rs_proc_list;	/* list of processes */
	struct proc_dir_entry	*rs_proc_dir;	/* dir entry under /proc/rk */
#endif
};

#define	NULL_RESOURCE_SET	((rk_resource_set_t)0)

extern	int		rk_resource_set_attach_reserve(rk_resource_set_t, rk_reserve_t);
extern	void		rk_resource_set_detach_reserve(rk_resource_set_t, rk_reserve_t);
/*
 * The following functions are defined as "inline" below.
 *	rk_resource_set_start_account
 *	rk_resource_set_stop_account
 */

/*
 * struct rk_reserve
 */
struct rk_reserve {
	/* */
	struct list_head	rsv_tmr_link;	/* link in timer */
	/* */
	struct list_head	rsv_rs_link;	/* link in resource set */
	rk_resource_set_t	rsv_rs;
	rk_reserve_type_t	rsv_type;	/* type */
	rk_reserve_state_t	rsv_state;	/* state */
	struct rk_reserve_ops	*rsv_ops;	/* rsv operation functions */
	void			*rsv_rsv;	/* link to real one */
	rk_timer_t		rsv_replenish_tmr;	/* replenishment timer */
	rk_reserve_param_data_t	rsv_reserve_param;	/* soft, firm, hard */
	/* */
#ifdef	linux
	struct proc_dir_entry	*rsv_proc_entry;	/* entry under /proc/rk/<rs_id>/ */
#endif
};
#define	NULL_RESERVE		((rk_reserve_t)0)

struct rk_reserve_ops {
	int			(*destroy) (rk_reserve_t);
	void			(*start_account) (rk_reserve_t);
	void			(*stop_account) (rk_reserve_t, cpu_tick_t);
	void			(*replenish) (rk_reserve_t, cpu_tick_t);
	void			(*enforce) (rk_reserve_t);
	void			(*attach)(rk_reserve_t, struct rs_proc_list*);
	void			(*detach)(rk_reserve_t, struct rs_proc_list*);
#ifdef	linux
	void			(*sleep_on) (rk_reserve_t);
	int			(*read_proc) (rk_reserve_t, char *);
#endif
};

extern	rk_reserve_t	rk_reserve_create(rk_resource_set_t, rk_reserve_type_t);
extern	int		rk_reserve_destroy(rk_reserve_t);

/*
 * timer for reserves:
 *	replenishment	... periodic
 *	expiration	... aperiodic
 *
 *   - tmr_expire:	holds an absolute time in CPU tick when its timer expires.
 *   - tmr_handler:	keeps the pointer to a handler funcion which is called 
 *			when its timer expires.
 *
 */
struct rk_timer {
	struct list_head	tmr_tmr;	/* list of timers */
	cpu_tick_data_t		tmr_expire;	/* in CPU cycles or ... */
	rk_timer_type_t		tmr_type;	/* timer type */
	/* */
	void			(*tmr_handler)(struct rk_timer*);
						/* handler */
	rk_reserve_t		tmr_rsv;	/* associated reserves */
};

#define	NULL_RK_TIMER		((rk_timer_t)0)

extern	void	rk_replenish_timer_create(rk_reserve_t, cpu_tick_t);
extern	void	rk_replenish_timer_destroy(rk_reserve_t);

extern	void	rk_enforce_timer_set_oneshot_mode(void);
extern	void	rk_enforce_timer_set_periodic_mode(void);

extern	void	rk_enforce_timer_start(rk_reserve_t, cpu_tick_t, cpu_tick_t);
extern	void	rk_enforce_timer_cancel(void);

/*
 * Global Variables and params
 */
extern	rk_resource_set_t	rk_current_resource_set;

extern	cpu_tick_data_t		rk_cpu_ticks_per_second;
extern	cpu_tick_data_t		rk_cpu_ticks_per_jiffy;
extern	cpu_tick_data_t	rk_cpu_tick_at_boot;		/* cpu tick when jiffies == 0 */
extern	cpu_tick_data_t	rk_cpu_tick_at_last_jiffy;	/* cpu tick at the last jiffy*/

#define RK_USECS_PER_JIFFY	(1000016L/HZ)		/* 10000 (HZ=100) */
#define RK_NANOSECS_PER_JIFFY	(1000016762L/HZ)	/* 10000167 (HZ=100) */

/*
 * There are the following conversion inline functions.
 * I don't know why, but MACROs didn't work for those purposes.
 */
/*
 * Conversions between tick and nanosec
 * rk_cpu_ticks_per_second:NANOSEC = ticks:nanosec
 */
static inline void
nanosec2tick(cpu_tick_t ns, cpu_tick_t tick)
{
	cpu_tick_data_t	nanosec = NANOSEC;
	/* *tick = (((*ns)*rk_cpu_ticks_per_second)+(nanosec/2))/nanosec;*/
	*tick = ((*ns)*rk_cpu_ticks_per_second)/nanosec;
}

/* 
 * Conversion between tick and usec
 * rk_cpu_ticks_per_second:MICROSEC = ticks:usec
 */
static inline void
tick2usec(cpu_tick_t tick, cpu_tick_t us)
{
	cpu_tick_data_t	msec = MICROSEC;
	/* *us = (((*tick)*msec)+(rk_cpu_ticks_per_second/2))/rk_cpu_ticks_per_second;*/
	*us = ((*tick)*msec)/rk_cpu_ticks_per_second;
}

static inline unsigned long
TICK2USEC(cpu_tick_t tick)
{
	cpu_tick_data_t	us;
	tick2usec(tick, &us);
	return (unsigned long)us;
}

static inline unsigned long
tick2jiffies(cpu_tick_t tick, cpu_tick_t last_tick)
{
	cpu_tick_data_t	d = (*tick)-(*last_tick);
	unsigned long	j;
	j = (d+rk_cpu_ticks_per_jiffy-1)/rk_cpu_ticks_per_jiffy;
#if 0
	printk("tick2jiffies: tick(%lu usec) last_tick(%lu usec) -> (%lu)\n",
	       TICK2USEC(tick), TICK2USEC(last_tick), j);
#endif
	j += jiffies;
	return j;
}

#define	TICK2JIFFIES(tick)	tick2jiffies(tick, &rk_cpu_tick_at_last_jiffy)
#define	TICK2JIFFIES2(tick,last)	tick2jiffies(tick,last)

/*
 * Enable/Disable RK hooks
 */
static void inline
rk_enable(void)
{
	extern void	rk_enable_schedule_cpu(void);
	extern void	rk_enable_isr(void);

	rk_enable_schedule_cpu();
	rk_enable_isr();
}

static void inline
rk_disable(void)
{
	extern void	rk_disable_schedule_cpu(void);
	extern void	rk_disable_isr(void);

	rk_disable_schedule_cpu();
	rk_disable_isr();
}

/*
 * Start/Stop accounting
 */
static inline void
rk_resource_set_start_account(rk_resource_set_t rs)
{
	rk_reserve_t	cpu = rs->rs_cpu;
	if (cpu) {
		cpu->rsv_ops->start_account(cpu);
	}
}

static inline void
rk_resource_set_stop_account(rk_resource_set_t rs, cpu_tick_t now)
{
	rk_reserve_t	cpu = rs->rs_cpu;
	if (cpu) {
		cpu->rsv_ops->stop_account(cpu, now);
	}
}

static inline void
rk_resource_set_do_enforcement(rk_resource_set_t rs)
{
	rk_reserve_t	cpu = rs->rs_cpu;
	if (cpu) {
		cpu->rsv_ops->enforce(cpu);
	}
}

static inline int
rk_resource_set_depleted(rk_resource_set_t rs)
{
	rk_reserve_t	cpu = rs->rs_cpu;
	if (cpu) {
		return (cpu->rsv_state & RSV_IS_DEPLETED);
	}
	return 0;
}
#else	/* __KERNEL__ */

#include <time.h>	/* for struct timespec */

/*
 * types for user programs
 */
typedef	void *			rk_resource_set_t;
typedef	void *			rk_reserve_t;

#endif	/* __KERNEL__ */

/*
 * function prototypes
 */
extern	rk_resource_set_t	rk_resource_set_create(void);
extern	int		rk_resource_set_destroy(rk_resource_set_t rs);

extern	rk_reserve_t	rk_cpu_reserve_create(rk_resource_set_t,
					      struct timespec *, struct timespec *,
					      rk_reserve_param_t);

extern	int		rk_cpu_reserve_ctl(rk_resource_set_t, rk_reserve_t,
					   struct timespec *, struct timespec *);

#ifdef	linux
extern	int		rk_resource_set_attach_process(rk_resource_set_t, pid_t);

extern	void		rk_proc_rs_create(rk_resource_set_t);
extern	void		rk_proc_rs_destroy(rk_resource_set_t);
extern	void		rk_proc_reserve_create(rk_reserve_t);
extern	void		rk_proc_reserve_destroy(rk_reserve_t);
#endif	/* linux */

#endif	/* RK_RK_H */
