#ifndef	RK_RK_LINUX_H
#define	RK_RK_LINUX_H

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/list.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <linux/slab.h>
#include <linux/param.h>
#include <asm/spinlock.h>

/*
 * Spin Lock & Unlock
 * Usage:
 *	{
 *		unsigned long flags;
 *
 *		rk_spin_lock(flags);
 *		<CRITICAL SECTION>
 *		rk_spin_unlock(flags);
 *	}
 */
extern	spinlock_t	rk_lock;
#define	rk_spin_lock(flags)	spin_lock_irqsave(&rk_lock, flags)
#define	rk_spin_unlock(flags)	spin_unlock_irqrestore(&rk_lock, flags)

/*
 *
 */
asmlinkage int sys_sched_setscheduler(pid_t, int, struct sched_param*);

struct rs_proc_list {
	struct list_head	rs_proc_list;
	pid_t			rs_proc_pid;
	struct task_struct	*rs_proc_task;
};

static inline void
rs_proc_list_apply(struct list_head *proc_list_head, void (*func)(struct rs_proc_list *))
{
	struct list_head	*proc_list;
	struct rs_proc_list	*rs_proc;

	proc_list = proc_list_head->next;
	while (proc_list != proc_list_head) {
		/* get a rs_proc */
		rs_proc = list_entry(proc_list, struct rs_proc_list, rs_proc_list);
		/* do it */
		func(rs_proc);

		/* next process */
		proc_list = proc_list->next;
	}
}

/*
 *
 */
#define	malloc(s)	kmalloc(s, SLAB_KERNEL)
#define	free(p)		kfree(p)
#define	bzero(p, s)	memset(p, 0, s)

/*
 *
 */
static inline
struct task_struct *
__rk_find_process_by_pid(pid_t pid)
{
	struct task_struct *tsk = current;

	if (pid)
		tsk = find_task_by_pid(pid);
	return tsk;
}

#if 0
/*
 * suspend & resume a task
 *		
 */
static inline void
rk_suspend_thread(struct task_struct *tsk)
{
#ifdef	DEBUG_RK
	printk("rk_suspend_thread: process(0x%x)\n", (int)tsk);
#endif

	tsk->state = TASK_STOPPED;
	tsk->need_resched = 1;
}

static inline void
rk_resume_thread(struct task_struct *tsk)
{
#ifdef	DEBUG_RK
	printk("rk_resume_thread: process(0x%x)\n", (int)tsk);
#endif

	tsk->need_resched = 0;
	wake_up_process(tsk);
}
#endif

#endif	/* RK_RK_LINUX_H */
