/*
 * reserve.c: generic reserve handling functions
 */
#include <rk/rk_linux.h>
#include <rk/rk_error.h>
#include <rk/rk.h>

void
rk_reserve_init(void)
{
	void	cpu_reserve_init(void);
	void	net_reserve_init(void);

	cpu_reserve_init();
#ifdef	RK_NET_RSV
	net_reserve_init();
#endif
}

rk_reserve_t
rk_reserve_create(rk_resource_set_t rs, rk_reserve_type_t type)
{
	rk_reserve_t	rsv;

	rsv = malloc(sizeof(struct rk_reserve));
	bzero(rsv, sizeof(struct rk_reserve));
#ifdef	DEBUG_RK
	printk("rk_reserve_create: rs(0x%x) type(0x%x) -> rsv(0x%x)\n",
	       (int)rs, (int)type, (int)rsv);
#endif
	INIT_LIST_HEAD(&rsv->rsv_rs_link);

	/* type is used below */
	rsv->rsv_type = type;
	rsv->rsv_rs = rs;

#ifdef	linux
	/* create entry as /proc/rk/<rs_id>/<rsv_id> */
	rk_proc_reserve_create(rsv);
#endif	/* linux */

	return rsv;
}

int
rk_reserve_destroy(rk_reserve_t rsv)
{
#ifdef	DEBUG_RK
	printk("rk_reserve_destroy: rsv(0x%x)\n", (int)rsv);
#endif
	/* if reserve has some capacity reserved */
	if (rsv->rsv_rsv) {
		return RK_ERROR;	/* error */
	}

#ifdef	linux
	/* remove entry under /proc/rk/<rs_id>
	 * must be before rk_resource_set_detach_reserve()
	 * since resource_set is reffered in rk_proc_reserve_destroy()
	 */
	rk_proc_reserve_destroy(rsv);
#endif	/* linux */

	/* if reserve is in a resource_set */
	if (rsv->rsv_rs) {
		rk_resource_set_detach_reserve(rsv->rsv_rs, rsv);
	}

	/* finally free it */
	bzero(rsv, sizeof(struct rk_reserve));	/* for sure */
	free(rsv);
	return RK_SUCCESS;		/* success */
}
