#include <rk/rk_linux.h>
#include <rk/rk_error.h>
#include <rk/rk.h>
#include <asm/system.h>
#include <asm/io.h>

#ifdef	DEBUG_RK
/**/
#define	DEBUG_RK_ISR
/**/
#ifdef	DEBUG_RK_ENFORCE
int	debug_rk_enforce_flag;
#endif
/**/
#endif	/*DEBUG_RK*/

/*
 * Hooks
 */
void	(*rk_kernel_intr_in_hook)(void);		/* arch/i386/kernel/irq.h */
void	(*rk_restore_all_hook)(struct pt_regs regs);	/* arch/i386/kernel/entry.S */
int	(*rk_ret_with_reschedule_hook)(struct pt_regs regs);

/*
 * For "rk_kernel_intr_in_hook":
 * void
 * rk_kernel_intr_in(void)
 * {
 * 	if (rk_do_IRQ()) {
 * 		__asm__("pushl $ret_from_intr\n\t"
 * 			"jmp do_IRQ");
 * 	} else {
 * 		__asm__ ("jmp ret_from_intr");
 * 	}
 * }
 *
 * This is called (is jumped to) while interrupt is disabled.
 */
void rk_kernel_intr_in(void);
__asm__(
	"\n" __ALIGN_STR"\n"
	"rk_kernel_intr_in:\n\t"
	"call rk_do_IRQ\n\t"
	"testl %eax,%eax\n\t"
	"je 1f\n\t"
	"pushl $ret_from_intr\n\t"
	"jmp do_IRQ\n\t"
	".align 4\n"
	"1:\n\t"
	"jmp ret_from_intr\n\t");

int
rk_do_IRQ(struct pt_regs regs)
{
	extern	int		rk_timer_isr(void);
	unsigned int		irq;
	int			ret = 1;	/* by default for Linux */

	irq = regs.orig_eax & 0xff;
	if (irq == 0) {
		/*
		 * return 1:	also for Linux 
		 * return 0:	only for RK
		 */
		ret = rk_timer_isr();
		if (!ret) {
			outb(0x20,0x20);	/* send EIO to PIC */
		}
	}
	return ret;
}

/*
 * For "rk_ret_with_reschedule_hook"
 */
asmlinkage int
rk_ret_with_reschedule(struct pt_regs regs)
{
	rk_resource_set_t	rs;
	rk_reserve_t		cpu_rsv;
	void cpu_reserve_sched_timeshare_self(void);

	if ((rs = rk_current_resource_set)) {
		if (rk_resource_set_depleted(rs)) {
#if	defined(DEBUG_RK_ISR) || defined(DEBUG_RK_ENFORCE)
			printk("rk_ret_with_reschedule: depleted rs(0x%x)\n", (int)rs);
#endif	/* DEBUG_RK_ENFORCE */
			cpu_rsv=rs->rs_cpu;
			switch (cpu_rsv->rsv_reserve_param.enf_mode) {
			case RSV_HARD:
				/* sleep_on only when hard enforcement */
				__sti();
				cpu_rsv->rsv_ops->sleep_on(cpu_rsv);
				return 1;
				break;
			case RSV_SOFT:
			case RSV_FIRM:
				__sti();
				cpu_reserve_sched_timeshare_self();
				return 0;
				break;
			}
		}
	}
	return 0;
}

/*
 * For "rk_restore_all_hook"
 */
asmlinkage void
rk_restore_all(struct pt_regs regs)
{
}

/*
 *
 */
void
rk_enable_isr(void)
{
	unsigned long flags;

	/*debug_rk_enforce_flag = 1;*/

	if (rk_kernel_intr_in_hook == rk_kernel_intr_in
	    && rk_restore_all_hook == rk_restore_all
	    && rk_ret_with_reschedule_hook == rk_ret_with_reschedule)
		return;

	rk_spin_lock(flags);
	{
		/* install hooks */
		rk_kernel_intr_in_hook = rk_kernel_intr_in;
		rk_restore_all_hook = rk_restore_all;
		rk_ret_with_reschedule_hook = rk_ret_with_reschedule;
		/* make a timer one shot mode */
		rk_enforce_timer_set_oneshot_mode();
	}
	rk_spin_unlock(flags);
#ifdef	DEBUG_RK
	printk("rk_enable_isr: rk kernel in/out hook enabled.\n");
#endif
}

void
rk_disable_isr(void)
{
	unsigned long flags;

	if (rk_kernel_intr_in_hook == (void*)0
	    && rk_restore_all_hook == (void*)0
	    && rk_ret_with_reschedule_hook == (void*)0)
		return;

	rk_spin_lock(flags);
	{
		/* remove hooks */
		rk_kernel_intr_in_hook = (void*)0;
		rk_restore_all_hook = (void*)0;
		rk_ret_with_reschedule_hook = (void*)0;
		/* make a timer the original setup */
		rk_enforce_timer_set_periodic_mode();
	}
	rk_spin_unlock(flags);
#ifdef	DEBUG_RK
	printk("rk_disable_isr: rk kernel in/out hook diabled.\n");
#endif
}


#ifdef	DEBUG_RK_ENFORCE
asmlinkage void
rk_ret_from_sys_call_msg(void) {
	unsigned long flags;
	rk_spin_lock(flags);
	{
		printk("ret_from_sys_call! pid(%d) eflag(0x%x)\n",
		       (int)current->pid, (int)flags);
	}
	rk_spin_unlock(flags);
}

asmlinkage void
rk_signal_return_msg(void) {
	unsigned long flags;
	rk_spin_lock(flags);
	{
		printk("signal_return! eflag(0x%x)\n", (int)flags);
	}
	rk_spin_unlock(flags);
}

asmlinkage void
rk_ret_from_intr_msg(void) {
	unsigned long flags;
	rk_spin_lock(flags);
	{
		printk("ret_from_intr! pid(%d) eflag(0x%x)\n",
		       (int)current->pid, (int)flags);
	}
	rk_spin_unlock(flags);
}

asmlinkage void
rk_reschedule_msg(void) {
	unsigned long flags;
	rk_spin_lock(flags);
	{
		printk("reschedule! pid(%d) eflag(0x%x)\n",
		       (int)current->pid, (int)flags);
	}
	rk_spin_unlock(flags);
}
#endif
