/*
 * rk_procfs.c (for Linux):
 *	implements the interface for procfs.
 *	shows the information on resource sets and reserves.
 *	
 */
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/stat.h>

#include <rk/rk_linux.h>
#include <rk/rk.h>

static struct proc_dir_entry *proc_rk_dir;

static int
proc_rk_status_read(char *page, char **start, off_t off,
		    int count, int *eof, void *data)
{
	char *p = page;
	extern int cpu_reserve_status_proc(char *);

	p += cpu_reserve_status_proc(p);

	return(p - page);	/* return the number of chars */
}

static int
proc_rs_status_read(char *page, char **start, off_t off,
		    int count, int *eof, void *data)
{
	char *p = page;
	extern int resource_set_status_proc(char *);

	p += resource_set_status_proc(p);

	return(p - page);	/* return the number of chars */
}

__initfunc(void rk_proc_init(void))
{
	struct proc_dir_entry *proc_rk_status;

	proc_rk_dir = create_proc_entry("rk", S_IFDIR, 0);

	proc_rk_status = create_proc_entry("status", S_IFREG|S_IRUGO, proc_rk_dir);
	proc_rk_status->read_proc = proc_rk_status_read;

	proc_rk_status = create_proc_entry("resource_set", S_IFREG|S_IRUGO, proc_rk_dir);
	proc_rk_status->read_proc = proc_rs_status_read;
}

void
rk_proc_rs_create(rk_resource_set_t rs)
{
	char	buf[16];

	sprintf(buf, "%x", (unsigned int)rs);
	rs->rs_proc_dir = create_proc_entry(buf, S_IFDIR, proc_rk_dir);
}

void
rk_proc_rs_destroy(rk_resource_set_t rs)
{
	char	buf[16];

	sprintf(buf, "%x", (unsigned int)rs);
	remove_proc_entry(buf, proc_rk_dir);
}

static int
proc_rsv_status_read(char *page, char **start, off_t off,
		     int count, int *eof, void *data)
{
	rk_reserve_t	rsv = (rk_reserve_t)data;
	return rsv->rsv_ops->read_proc(rsv, page);
}

void
rk_proc_reserve_create(rk_reserve_t rsv)
{
	rk_resource_set_t	rs = rsv->rsv_rs;
	char	buf[16];

#ifdef	DEBUG_RK
	printk("rk_proc_reserve_create: rs(0x%x) rsv(0x%x) type(0x%x)\n",
	       (int)rs, (int)rsv, (int)rsv->rsv_type);
#endif

	switch(rsv->rsv_type) {
	case RSV_CPU:
		sprintf(buf, "cpu");
		break;
	default:
		sprintf(buf, "%x", (unsigned int)rsv);
		break;
	}

	rsv->rsv_proc_entry = create_proc_entry(buf, S_IFREG|S_IRUGO,
						rs->rs_proc_dir);

	rsv->rsv_proc_entry->read_proc = proc_rsv_status_read;
	rsv->rsv_proc_entry->data = rsv;
}

void
rk_proc_reserve_destroy(rk_reserve_t rsv)
{
	rk_resource_set_t	rs = rsv->rsv_rs;
	char	buf[16];

#ifdef	DEBUG_RK
	printk("rk_proc_reserve_destroy: rs(0x%x) rsv(0x%x) type(0x%x)\n",
	       (int)rs, (int)rsv, (int)rsv->rsv_type);
#endif

	remove_proc_entry(buf, rs->rs_proc_dir);
}
