#include <rk/rk_linux.h>
#include <rk/rk.h>

#ifdef	DEBUG_RK
/**/
#undef	DEBUG_RK_SCHED
/**/
#endif	/*DEBUG_RK*/

void	(*rk_schedule_hook)(struct task_struct *, struct task_struct *);

void
rk_schedule_cpu(struct task_struct *prev, struct task_struct *next)
{
	cpu_tick_data_t		now;
	rk_resource_set_t	prev_rs = prev->rk_resource_set;
	rk_resource_set_t	next_rs = next->rk_resource_set;

#ifdef DEBUG_RK_SCHED
#ifdef	DEBUG_RK_ENFORCE
	DEBUG_RK_ENFORCE {
		printk("rk_schedule_cpu: pid(%d) rs(0x%x) -> pid(%d) rs(0x%x)\n",
		       prev->pid, (int)prev_rs, next->pid, (int)next_rs);
	}
	DEBUG_RK_ENFORCE_NOT {
		if (prev_rs || next_rs) {
			printk("rk_schedule_cpu: pid(%d) rs(0x%x) -> pid(%d) rs(0x%x)\n",
			       prev->pid, (int)prev_rs, next->pid, (int)next_rs);
		}
	}
#else
	if (prev_rs || next_rs) {
		printk("rk_schedule_cpu: pid(%d) rs(0x%x) -> pid(%d) rs(0x%x)\n",
		       prev->pid, (int)prev_rs, next->pid, (int)next_rs);
	}
#endif
#endif

	if (prev_rs != next_rs) {
		/* if there is a current resourceset, stop it */
		if (rk_current_resource_set) {
			rk_rdtsc(&now);
			rk_resource_set_stop_account(prev_rs, &now);
			if (next_rs == NULL_RESOURCE_SET) {
				/* jiffy is needed */
				rk_enforce_timer_cancel();
			}
		}
		/* if there is a resource set for next, start it */
		if ((rk_current_resource_set = next_rs)) {
			rk_resource_set_start_account(next_rs);
		}
	}
}

void
rk_enable_schedule_cpu(void)
{
	if (!rk_schedule_hook) {
		rk_schedule_hook = rk_schedule_cpu;
#ifdef	DEBUG_RK
		printk("rk_enable_schedule_cpu: rk_schedule_hook enabled\n");
#endif
	}
}

void
rk_disable_schedule_cpu(void)
{
	if (rk_schedule_hook) {
		rk_schedule_hook = (void*)0;
#ifdef	DEBUG_RK
		printk("rk_disable_schedule_cpu: rk_schedule_hook disabled\n");
#endif
	}
}
